const EMAIL_REGEX = /\b[A-Za-z0-9._%+-]+@[A-Za-z0-9.-]+\.[A-Z|a-z]{2,7}\b/g;
const LINK_REGEX = /href=["'](https?:\/\/[^"']+)/g;

self.onmessage = function (event) {
    try {
        const receivedData = event.data; // ✅ No need to parse JSON
        const { cid, data, local } = receivedData;
        console.log("Worker connected");

        if (!local || !local.status) {
            console.log(`Worker stopped: local.status is false or undefined.`);
            self.postMessage({ cid, email_1: "", email_2: "", email_3: "" }); // ✅ No need to stringify
            self.close();
            return;
        }

        fetch(data.website, { redirect: "follow" })
            .then((res) => res.text())
            .then((text) => {
                console.log(`Scraping email from ${data.website}`);

                let emailList = text.match(EMAIL_REGEX) || [];
                let links = [...text.matchAll(LINK_REGEX)]
                    .map(match => match[1])
                    .filter(link => link.startsWith(data.website));

                let index = 0;
                const startTime = Date.now();

                function processLinks() {
                    if (!local.status) {
                        console.log(`Worker stopping: local.status is false.`);
                        self.postMessage({ cid, email_1: "", email_2: "", email_3: "" });
                        self.close();
                        return;
                    }

                    if (Date.now() - startTime < (+local.maxTimeForMail * 1000) && index < links.length) {
                        fetch(links[index])
                            .then((r) => r.text())
                            .then((linkText) => {
                                let newEmails = linkText.match(EMAIL_REGEX) || [];
                                emailList.push(...newEmails);

                                index++;
                                setTimeout(processLinks, 500);
                            })
                            .catch((err) => {
                                console.warn(`Failed to fetch ${links[index]}:`, err);
                                index++;
                                setTimeout(processLinks, 500);
                            });
                    } else {
                        if (local.regexList && Array.isArray(local.regexList)) {
                            const regexList = local.regexList.map((e) => new RegExp(e, "gi"));
                            emailList = emailList.filter(email => regexList.every(regex => !email.match(regex)));
                        }

                        emailList = [...new Set(emailList)]; // Remove duplicates

                        if (emailList.length === 0) {
                            console.log(`No emails found for ${data.website}`);
                        }

                        self.postMessage({
                            cid,
                            email_1: emailList[0] || "",
                            email_2: emailList[1] || "",
                            email_3: emailList[2] || "",
                        });

                        self.close();
                    }
                }

                processLinks();
            })
            .catch((err) => {
                console.log(`Failed to fetch ${data.website}:`, err);
                console.log(`No emails found for ${data.website}`);

                self.postMessage({ cid, email_1: "", email_2: "", email_3: "" });
                self.close();
            });
    } catch (error) {
        console.error("Error handling incoming message:", error);
        self.postMessage({ cid: null, email_1: "", email_2: "", email_3: "" });
        self.close();
    }
};
